@@ -21,10 +21,28 @@ test_that("sql_lib basic test", {
         table = "mtcars",
         where_fields = "gear",
         where_values = 4)
+
+
+    # As vector casted to data.frames makes rows, next statement throws an error
+    expect_error(rsql$gen_select(
+        select_fields = c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am"),
+        table = "mtcars",
+        where_fields = c("gear", "carb"),
+        where_values = c(4, 1)))
+
+    query_sql <- rsql$gen_select(
+        select_fields = c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am"),
+        table = "mtcars",
+        where_values = data.frame(gear = 4))
     mtcars.observed <- rsql$execute_select(query_sql)
     expect_equal(nrow(mtcars.observed), 12)
 })
 
+test_that("util", {
+    expect_equal(rm_vector_quotes(c("hel'lo", "'hel'lo'", "'tell''"))
+                 , c("hel''lo", "hel''lo", "tell''"))
+})
+
 test_that("legal entities", {
 
     #dbWriteTable(rsql$conn, name = "mtcars", mtcars, overwrite = TRUE)
@@ -61,14 +79,13 @@ test_that("sql_lib insert and delete test", {
     update_fields <- c("mpg", "cyl")
     update_values <- data.frame(1, 2)
 
-    where_fields <- "disp"
-    where_values <- 258.0
+    where_values <- data.frame(disp = 258.0)
     update_sql <- rsql$gen_update(table = "mtcars",
                                   update_fields = update_fields, values = update_values,
-                                  where_fields = where_fields, where_values = where_values)
+                                  where_values = where_values)
     rsql$execute_update(update_sql)
 
-    delete.sql <- rsql$gen_delete("mtcars", c("mpg"), c("1"))
+    delete.sql <- rsql$gen_delete("mtcars", where_values = data.frame(mpg = 1))
     rsql$execute_delete(delete.sql)
 })
 
@@ -84,9 +101,7 @@ test_that("sql_lib select, insert and delete with dataframe", {
     update_fields <- c("mpg", "cyl")
     update_values <- data.frame(1, 2)
     names(update_values) <- update_fields
-    where_fields <- "disp"
-    where_values <- 275.8
-    names(where_values) <- where_fields
+    where_values <- data.frame(disp = 275.8)
     update_sql <- rsql$gen_update("mtcars", values = update_values,
                                   where_values = where_values)
     rsql$execute_update(update_sql)
@@ -98,15 +113,69 @@ test_that("sql_lib select, insert and delete with dataframe", {
     check_df <- rsql$execute_select(check_sql)
     expect_equivalent(check_df %>% dplyr::group_by(mpg, cyl), update_values)
 
-    delete.sql <- rsql$gen_delete("mtcars", c("mpg"), c("1"))
+    delete.sql <- rsql$gen_delete("mtcars", where_values = data.frame(mpg = 1))
     rsql$execute_delete(delete.sql)
 
 })
 
+
+test_that("sql_lib select, insert, update and delete with dataframe and stuffed apostrophe", {
+    mtcars$model <- rownames(mtcars)
+    dbWriteTable(rsql$conn, name = "mtcars", value = mtcars, overwrite = TRUE)
+    insert_fields <- c("mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am",
+                       "gear", "carb")
+    insert_data <- data.frame(matrix(1: ( 11 * 7 ), ncol = 11, byrow = TRUE))
+    names(insert_data) <- insert_fields
+    insert_data$model <- paste("'car", 1:7, "'", sep = "")
+    insert_data[7, "model"] <- "'O'Connor'"
+    insert.sql <- rsql$gen_insert("mtcars", values_df = insert_data)
+    rsql$execute_insert(insert.sql)
+
+    update_fields <- c("mpg", "cyl", "model")
+    update_values <- data.frame(1, 2, "'O'Connor'")
+    names(update_values) <- update_fields
+    where_values_df <- data.frame(disp = 275.8)
+    update_sql <- rsql$gen_update("mtcars", values = update_values,
+                                  where_values = where_values_df)
+    rsql$execute_update(update_sql)
+
+    where.accent <- "áccent"
+    #where.accent <- "Bélgica"
+    where.df <- data.frame(model = where.accent, stringsAsFactors = FALSE)
+    exists.sql <- rsql$gen_select("id_pais", table = "pais",
+                                  where_values = where.df)
+    exists.sql
+
+
+    where_values.df <- data.frame(model = "'O'Connor'")
+
+    check_sql <- rsql$gen_select(update_fields,
+                                 table = "mtcars",
+                                 where_values = where_values.df,
+                                 distinct = TRUE)
+    expect_equal(check_sql, "select distinct mpg, cyl, model from mtcars where (model) in ('O''Connor')")
+    check_df <- rsql$execute_select(check_sql)
+    observed_values <- check_df %>% dplyr::filter(mpg == 1)
+    observed_values$model <- add_quotes(observed_values$model)
+    expect_equivalent(observed_values, update_values)
+
+    # With or without quotes
+    delete.where.df <- data.frame(model = "O'Connor")
+    delete.sql <- rsql$gen_delete("mtcars", where_values = delete.where.df)
+    expect_equal(delete.sql, "delete from mtcars where (model) in ('O''Connor')")
+    delete.where.df <- data.frame(model = "'O'Connor'")
+    #delete.where.df <- data.frame(model = "'O'Connor'")
+    delete.sql <- rsql$gen_delete("mtcars", where_values = delete.where.df)
+    expect_equal(delete.sql, "delete from mtcars where (model) in ('O''Connor')")
+    rsql$execute_delete(delete.sql)
+    check_df <- rsql$execute_select(check_sql)
+    expect_equal(nrow(check_df), 0)
+
+})
+
 test_that("sql_lib select with where clause", {
     query_sql <- rsql$gen_select(table = "mtcars", select_fields = "*",
-                                where_fields = c("mpg"),
-                                where_values = 21)
+                                where_values = data.frame(mpg = 21))
     selected.mtcars <- rsql$execute_select(query_sql)
     expect_equal(selected.mtcars$drat, c(3.9, 3.9))
 })
@@ -140,6 +209,11 @@ test_that("retrieveInsert", {
                       field_id = "vehicle_id")
     expect_true(is.na(vehicle.id.observed))#
 
+    vehicle.id.observed <-
+        rsql$retrieve(table = "retrieveInsert", values_uk = values.uk,
+                      field_id = "vehicle_id")
+    expect_true(is.na(vehicle.id.observed))#
+
 
     vehicle_id <- data.frame(vehicle_id = 1)
     #Update vehicle_id to 1
@@ -154,7 +228,7 @@ test_that("update symbols", {
     update.pk <- data.frame(pk = 1, stringsAsFactors = FALSE)
     update.values <- data.frame(field.1 = "a", field.2 = "b", field.3 = NA, stringsAsFactors = FALSE)
     observed.update <- sql_gen_update("foo", values = update.values, where_values = update.pk)
-    expect_equal(observed.update, "update foo set (field.1,field.2,field.3)=('a','b',NULL) where (pk) in  ( 1 )")
+    expect_equal(observed.update, "update foo set (field.1,field.2,field.3)=('a','b',NULL) where (pk) in ('1')")
 })
 
 rsql$disconnect()
